from ..Utils import utils
utils = utils()

from .Towns import towns

class OnlinePlayer:
    def __init__(self, name="", world="", x=0, y=0, z=0):
        self.name = name
        self.world = world
        self.x = x
        self.y = y
        self.z = z
    def __repr__(self):
        return "Name: %s \nWorld: %s \nX: %s \nY: %s \nZ: %s" % (self.name, self.world, self.x, self.y, self.z)

class players:
    def all(self): return self.townless.all() + self.residents.all()
    def __init__(self, map): 
        self.online = self.online(map)
        self.residents = self.residents(map)
        self.townless = self.townless(map)

    class residents:
        def __init__(self, map): 
            self.mapName = map
            self.towns = towns(map)
        def all(self): 
            output = []

            for t in self.towns.all():
                for res in t.residents:
                    output.append(res)

            return output

    class townless:
        def __init__(self, map): 
            self.mapName = map
            self.playerData = utils.playerData(map)
        def all(self): return []

    class online:
        def __init__(self, map): 
            self.mapName = map
            self.data = utils.playerData(map)
        def all(self):
            output = []
            
            for player in self.data['players']:
                currentPlayer = OnlinePlayer(player["account"], player["world"], player["x"], player["y"], player["z"])
                output.append(currentPlayer)
            
            return output
        def find(self, playerName): return self.get(playerName)
        def get(self, playerName, players=None):
            if players is None: players = self.all()
            foundPlayer = utils.find(lambda player: player.name == playerName, players)
        
            if foundPlayer is None: return "Could not find player '" + playerName + "'" 
            return foundPlayer


from ..Utils import FetchError, utils
utils = utils()

class Town:
    def __init__(self, name="", nation="", mayor="", residents=[], area=0, x=0, z=0):
        self.name = name
        self.nation = nation
        self.mayor = mayor
        self.residents = residents
        self.area = area
        self.x = x
        self.z = z
    def __repr__(self):
        return "Name: %s \nNation: %s \nMayor: %s \nResidents: %s \nArea: %s \nX: %s \nZ: %s" % (self.name, self.nation, self.mayor, self.residents, self.area, self.x, self.z)
        
class towns:
    def __init__(self, map): self.mapName = map
    def all(self):
        townsArray, areas = []
        mapData = None
        
        try: 
            mapData = utils.mapData(self.mapName)

            if mapData is not None: areas = mapData["sets"]['townyPlugin.markerset']["areas"]
            else: raise FetchError
        except FetchError: print("Error fetching map data! Type is 'None'")

        townAreaNames = list(areas.keys())

        for i in range(len(townAreaNames)):
            town = areas[townAreaNames[i]]
            rawinfo = town["desc"].split("<br />")

            info = []

            for x in rawinfo:
                info.append(utils.striptags(x))

            townName = info[0].split(" (")[0].strip()         
            if townName.endswith("(Shop)"): continue
        
            mayor = info[1][7:]
            if mayor == "": continue

            nationName = info[0].split(" (")[1][0:-1] if "" else info[0].split(" (")[1][0:-1].strip()
            residents = info[2][9:].split(", ")

            area = utils.calcArea(town["x"], town["z"], len(town["x"]), 256)

            x = round((max(town["x"]) + min(town["x"])) / 2)
            z = round((max(town["z"]) + min(town["z"])) / 2)

            currentTown = Town(townName, nationName, mayor, residents, area, x, z)
            townsArray.append(currentTown)

        return townsArray
    def get(self, townName, towns=None):
        if towns is None: towns = self.all()
        foundTown = utils.find(lambda town: town.name == townName, towns)

        if foundTown is None: return "Could not find town '" + townName + "'" 
        return foundTown


from ..Utils import utils
utils = utils()

class Nation:
    def __init__(self, name="", king="", capital="", residents=[], towns=[], area=0):
        self.name = name
        self.king = king
        self.capital = capital
        self.residents = residents
        self.towns = towns
        self.area = area
    def __repr__(self):
        return "Name: %s \nKing: %s \nCapital: %s \nResidents: %s \nTowns: %s \n" % (self.name, self.king, self.capital, self.residents, self.towns)
        
class nations:
    def __init__(self, map): 
        self.mapName = map
        self.mapData = utils.mapData(map)
    def all(self): return NotImplementedError


# Read contents README
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

from setuptools import setup
setup(
    name="EarthMC",
    version="2.2.1",
    description="Provides data on people, places and more on the EarthMC Minecraft server.",
    author="Owen3H",
    license="MIT",
    url="https://github.com/EarthMC-Toolkit/EarthMC-Py",
    py_modules=["__init__", "Utils"],
    package_dir={'': 'src'},
    long_description=long_description,
    long_description_content_type="text/markdown"
)

# Build dist using this: python setup.py sdist bdist_wheel
# Upload using this: python -m twine upload dist/*